//
//  OOSPreSignedURLTests.m
//  OOSTests
//
//  Created by Ye Tong on 2018/11/8.
//  Copyright © 2018年 CTYun. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "OOSPreSignedURL.h"

@interface OOSPreSignedURLTests : XCTestCase

@end

@implementation OOSPreSignedURLTests

NSString *const AccessKey2 = @"2510935f75d781385b35";
NSString *const SecretKey2 = @"567b89b544a89bb3d3ccef4be8507e6d30d75579";

+ (void)setUp {
	
	OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:AccessKey2 secretKey:SecretKey2];
	OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionHangZhou
																		 credentialsProvider:provider];
	[OOSServiceManager defaultServiceManager].defaultServiceConfiguration = configuration;
}

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testDefault {
	OOSPreSignedURLBuilder *builder = [OOSPreSignedURLBuilder defaultPreSignedURLBuilder];
	XCTAssertNotNil(builder);
}

- (void)testRegister {
	
	OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:AccessKey2 secretKey:SecretKey2];
	
	OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionBeiJing2
																		 credentialsProvider:provider];
	
	[OOSPreSignedURLBuilder registerPreSignedURLBuilderWithConfiguration:configuration forKey:@"bj2"];
	
	OOSPreSignedURLBuilder *builder = [OOSPreSignedURLBuilder PreSignedURLBuilderForKey:@"bj2"];
	XCTAssertNotNil(builder);
	
	[OOSPreSignedURLBuilder removePreSignedURLBuilderForKey:@"bj2"];
}

- (void)testCustomServiceConfiguration {
	
	OOSPreSignedURLBuilder *builder = [OOSPreSignedURLBuilder defaultPreSignedURLBuilder];
	
	OOSGetPreSignedURLRequest *getPreSignedURLRequest = [OOSGetPreSignedURLRequest new];
	getPreSignedURLRequest.bucket = @"norris";
	getPreSignedURLRequest.key = @"onepiece.jpg";
	getPreSignedURLRequest.HTTPMethod = OOSHTTPMethodGET;
	getPreSignedURLRequest.expires = [NSDate dateWithTimeIntervalSinceNow:3600];
	getPreSignedURLRequest.limitrate = @(2048);
	
	[[[builder getPreSignedURL:getPreSignedURLRequest] continueWithBlock:^id(OOSTask *task) {
		
		NSURL *presignedURL = task.result;
		NSLog(@"presignedURL = %@", presignedURL);
		XCTAssertNotNil(presignedURL);
		
		NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:presignedURL];
		request.cachePolicy = NSURLRequestReloadIgnoringLocalCacheData;

		dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
		[[[NSURLSession sharedSession] dataTaskWithRequest:request
										 completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
											 XCTAssertNil(error, @"response contains error:%@,\n presigned URL is:%@",error,presignedURL.absoluteString);

											 if (((NSHTTPURLResponse *)response).statusCode < 200 || ((NSHTTPURLResponse *)response).statusCode >=300) {
												 XCTFail(@"response status Code is :%ld", (long)((NSHTTPURLResponse *)response).statusCode);
												 NSLog(@"response data is:%@",[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
											 }

											 XCTAssertEqual(23416, [data length],
															@"received object is different from sent object. expect file size:%lu, got:%lu", (unsigned long)65961003, (unsigned long)[data length]);

											 dispatch_semaphore_signal(semaphore);
										 }] resume];

		XCTAssertEqual(dispatch_semaphore_wait(semaphore, dispatch_time(DISPATCH_TIME_NOW, 240 * NSEC_PER_SEC)), 0);
		
		return nil;
	}] waitUntilFinished];
	
}

@end
