//
//  OOSServiceTests.m
//  OOSTests
//
//  Created by Ye Tong on 2018/11/4.
//  Copyright © 2018年 CTYun. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "OOSService.h"

@interface OOSServiceTests : XCTestCase

@end

@implementation OOSServiceTests

NSUInteger const OOSTest256KB = 1024 * 256;
NSUInteger const OOSTestsTransferManagerMinimumPartSize = 5 * 1024 * 1024;
NSString *const OOSTestBucketNamePrefix = @"ios-v2-test-";
NSString *const TestAttrBucketName = @"norris-attr";
NSString *const TestBucketName = @"norris";
static NSString *testBucketNameGeneral = nil;
static NSMutableArray<NSString *> *testBucketsCreated;

+ (void)setUp {
	testBucketsCreated = [NSMutableArray new];
	
	//Create bucketName
	NSTimeInterval timeIntervalSinceReferenceDate = [NSDate timeIntervalSinceReferenceDate];
	testBucketNameGeneral = [NSString stringWithFormat:@"%@%lld", OOSTestBucketNamePrefix, (int64_t)timeIntervalSinceReferenceDate];
	
	OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:@"2510935f75d781385b35" secretKey:@"567b89b544a89bb3d3ccef4be8507e6d30d75579"];
	OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionHangZhou
																		 credentialsProvider:provider];
	[OOSServiceManager defaultServiceManager].defaultServiceConfiguration = configuration;
	
	
	[OOSDDLog sharedInstance].logLevel = OOSDDLogLevelAll;
	[OOSDDLog addLogger:[OOSDDOSLogger sharedInstance]];
}

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testDefault {

	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	XCTAssertNotNil(client.configuration);
	XCTAssertEqual(client.configuration.regionType, OOSRegionHangZhou);
	
	OOSTask *task = client.configuration.credentialsProvider.credentials;
	OOSCredentials *credentials = task.result;
	XCTAssertNotNil(credentials.accessKey);
	XCTAssertNotNil(credentials.secretKey);
}

- (void)testRegister {
	
	OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:@"2510935f75d781385b35" secretKey:@"567b89b544a89bb3d3ccef4be8507e6d30d75579"];
	
	OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionBeiJing2
																			 credentialsProvider:provider];
	
	[OOS registerWithConfiguration:configuration forKey:@"bj2"];
	
	OOS *client = [OOS ForKey:@"bj2"];
	XCTAssertNotNil(client.configuration);
	XCTAssertEqual(client.configuration.regionType, OOSRegionBeiJing2);
}

- (void)testListBucket {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	[[[client listBuckets:[OOSRequest new]] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListBucketsOutput class]],@"The response object is not a class of [%@]", NSStringFromClass([OOSListBucketsOutput class]));
		
		OOSListBucketsOutput *listBucketOutput = task.result;
		NSLog(@" testListBucket ========= responseObject is: ================  %@", [listBucketOutput description]);
		return nil;
	}] waitUntilFinished];
}

- (void)testHeadBucket {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSHeadBucketRequest *headBucketReq = [OOSHeadBucketRequest new];
	headBucketReq.bucket = @"norris";
	
	[[[[client headBucket:headBucketReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);

		OOSHeadBucketRequest *headBucket2Req = [OOSHeadBucketRequest new];
		headBucket2Req.bucket = @"xxxxx";
		
		return [client headBucket:headBucket2Req];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNotNil(t.error, @"The request failed. error: [%@]", t.error);
		
		return nil;
	}] waitUntilFinished];
}

- (void)testBucketCors {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSGetBucketCorsRequest *getBucketCorsReq = [OOSGetBucketCorsRequest new];
	getBucketCorsReq.bucket = TestAttrBucketName;
	
	[[[[[client getBucketCors:getBucketCorsReq] continueWithBlock:^id(OOSTask *task) {
		NSMutableArray *corsRules = [NSMutableArray new];
		
		OOSCORSRule *rule = [OOSCORSRule new];
		rule.identifier = @"A";
		rule.allowedMethods = @[@"PUT",@"POST",@"DELETE"];
		rule.allowedOrigins = @[@"*"];
		[corsRules addObject:rule];
		
		OOSCORSRule *rule2 = [OOSCORSRule new];
		rule2.identifier = @"B";
		rule2.allowedOrigins = @[@"*"];
		rule2.allowedMethods = @[@"GET"];
		rule2.allowedHeaders = @[@"*"];
		[corsRules addObject:rule2];
		
		OOSCORSConfiguration *cors = [OOSCORSConfiguration new];;
		cors.CORSRules = corsRules;
		
		OOSPutBucketCorsRequest *putBucketCorsReq = [OOSPutBucketCorsRequest new];
		putBucketCorsReq.bucket = getBucketCorsReq.bucket;
		putBucketCorsReq.CORSConfiguration = cors;
		
		return [client putBucketCors:putBucketCorsReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error, @"The request failed. error: [%@]", t.error);
		
		OOSDeleteBucketCorsRequest *deleteBucketCorsReq = [OOSDeleteBucketCorsRequest new];
		deleteBucketCorsReq.bucket = getBucketCorsReq.bucket;
		
		return [client deleteBucketCors:deleteBucketCorsReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error, @"The request failed. error: [%@]", t.error);
		
		return nil;
	}] waitUntilFinished];
}

- (BOOL)isContainBucketName:(NSString *)bucketName inBucketArray:(NSArray *)bucketsArray {
	for (OOSBucket* bucket in bucketsArray) {
		if ([bucket.name isEqualToString:bucketName]) return YES;
	}
	return NO;
}

- (void)testCreateDeleteBucket {
	NSString *bucketNameTest2 = testBucketNameGeneral;
	[testBucketsCreated addObject:bucketNameTest2];
	
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSCreateBucketRequest *createBucketReq = [OOSCreateBucketRequest new];
	createBucketReq.ACL = OOSBucketCannedACLPublicReadWrite;
	createBucketReq.bucket = bucketNameTest2;
	
	[[[[[[client createBucket:createBucketReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return [client listBuckets:[OOSRequest new]];
	}] continueWithBlock:^id(OOSTask *task) {
		//Check if bucket are there.
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListBucketsOutput class]],@"The response object is not a class of [%@]", NSStringFromClass([OOSListBucketsOutput class]));
		
		OOSListBucketsOutput *listBucketOutput = task.result;
		XCTAssertTrue([self isContainBucketName:bucketNameTest2 inBucketArray:listBucketOutput.buckets], @"%@ can not be found in  Bucket List",bucketNameTest2);
		
		OOSDeleteBucketRequest *deleteBucketReq = [OOSDeleteBucketRequest new];
		deleteBucketReq.bucket = bucketNameTest2;
		
		return [client deleteBucket:deleteBucketReq];
	}] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return [client listBuckets:[OOSRequest new]];
	}] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListBucketsOutput class]],@"The response object is not a class of [%@]", NSStringFromClass([OOSListBucketsOutput class]));
		
		OOSListBucketsOutput *listBucketOutput = task.result;
		XCTAssertFalse([self isContainBucketName:bucketNameTest2 inBucketArray:listBucketOutput.buckets], @"%@ should NOT befound in  Bucket List",bucketNameTest2);
		return nil;
	}] waitUntilFinished];
}

- (void)testGetBucketAcl {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSGetBucketAclRequest *getBucketAclReq = [OOSGetBucketAclRequest new];
	getBucketAclReq.bucket = TestAttrBucketName;
	
	[[[client getBucketAcl:getBucketAclReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSGetBucketAclOutput class]]);
		
		return nil;
	}] waitUntilFinished];
}

- (void)testListObjects {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSListObjectsRequest *listObjectsReq = [OOSListObjectsRequest new];
	listObjectsReq.bucket = @"norris";
	NSString *prefix = @"onepiece";
	listObjectsReq.prefix = prefix;
	listObjectsReq.maxKeys = [NSNumber numberWithInt:2];
	
	[[[client listObjects:listObjectsReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListObjectsOutput class]]);
		
		OOSListObjectsOutput *listObjectsOutput = task.result;
		if ( [[listObjectsOutput contents] count] > 0 ){
			OOSObject *obj1 = [[listObjectsOutput contents] firstObject];
			XCTAssertTrue([obj1.key hasPrefix:prefix]);
		}
		
		return nil;
	}] waitUntilFinished];
}

- (void)testBucketLifeCycle {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSLifecycleExpiration *expire = [OOSLifecycleExpiration new];
	expire.days = @10;
	
	OOSRule *myRule = [OOSRule new];
	myRule.prefix = @"key-prefix";
	myRule.status = OOSExpirationStatusEnabled;
	myRule.expiration = expire;
	
	OOSLifecycleConfiguration *cycleConf = [OOSLifecycleConfiguration new];
	cycleConf.rules = @[myRule];
	
	OOSPutBucketLifecycleRequest *putBucketLifeCycleReq = [OOSPutBucketLifecycleRequest new];
	putBucketLifeCycleReq.bucket = TestAttrBucketName;
	putBucketLifeCycleReq.lifecycleConfiguration = cycleConf;
	
	[[[[[client putBucketLifecycle:putBucketLifeCycleReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		
		OOSGetBucketLifecycleRequest *getBucketLifeCycle = [OOSGetBucketLifecycleRequest new];
		getBucketLifeCycle.bucket = putBucketLifeCycleReq.bucket;
		
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:30]];
		return [client getBucketLifecycle:getBucketLifeCycle];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		XCTAssertTrue([t.result isKindOfClass:[OOSGetBucketLifecycleOutput class]]);
		
		OOSGetBucketLifecycleOutput *lifecycleOutput = t.result;
		OOSRule *firstRule = [lifecycleOutput.rules firstObject];
		XCTAssertTrue([firstRule.prefix isEqualToString:@"key-prefix"]);
		
		OOSDeleteBucketLifecycleRequest *deleteBucketLifecycleReq = [OOSDeleteBucketLifecycleRequest new];
		deleteBucketLifecycleReq.bucket = putBucketLifeCycleReq.bucket;
		
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:30]];
		return [client deleteBucketLifecycle:deleteBucketLifecycleReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return nil;
	}] waitUntilFinished];
}

- (void)testBucketWebsite {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);

	OOSWebsiteConfiguration *config = [OOSWebsiteConfiguration new];
	
	OOSErrorDocument *errorDocument = [OOSErrorDocument new];
	errorDocument.key = @"error.html";
	config.errorDocument = errorDocument;
	
	OOSIndexDocument *indexDocument = [OOSIndexDocument new];
	indexDocument.suffix = @"index.html";
	config.indexDocument = indexDocument;
	
	OOSPutBucketWebsiteRequest *putReq = [OOSPutBucketWebsiteRequest new];
	putReq.bucket = TestAttrBucketName;
	putReq.websiteConfiguration = config;
	
	[[[[[client putBucketWebsite:putReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		
		OOSGetBucketWebsiteRequest *getReq = [OOSGetBucketWebsiteRequest new];
		getReq.bucket = TestAttrBucketName;
		
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:30]];
		return [client getBucketWebsite:getReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		XCTAssertTrue([t.result isKindOfClass:[OOSGetBucketWebsiteOutput class]]);

		OOSGetBucketWebsiteOutput *websiteOutput = t.result;
		XCTAssertTrue([websiteOutput.errorDocument.key containsString:@"error.html"]);
		XCTAssertTrue([websiteOutput.indexDocument.suffix containsString:@"index.html"]);

		OOSDeleteBucketWebsiteRequest *deleteReq = [OOSDeleteBucketWebsiteRequest new];
		deleteReq.bucket = putReq.bucket;

		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:30]];
		return [client deleteBucketWebsite:deleteReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return t;
	}] waitUntilFinished];
}

- (void)testBucketLogging {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSLoggingEnabled *enabled = [OOSLoggingEnabled new];
	enabled.targetBucket = @"norris";
	enabled.targetPrefix = @"nnn";
	enabled.triggerTargetBucket = @"norris";
	enabled.triggerTargetPrefix = @"mmm";
	enabled.triggerSourceBucket = @"norris";
	enabled.triggerSourcePrefix = @"bbb";
	
	OOSBucketLoggingStatus *status = [OOSBucketLoggingStatus new];
	status.loggingEnabled = enabled;
	
	OOSPutBucketLoggingRequest *putReq = [OOSPutBucketLoggingRequest new];
	putReq.bucket = TestAttrBucketName;
	putReq.bucketLoggingStatus = status;

	[[[[client putBucketLogging:putReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);

		OOSGetBucketLoggingRequest *getReq = [OOSGetBucketLoggingRequest new];
		getReq.bucket = TestAttrBucketName;

		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:30]];
		return [client getBucketLogging:getReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		XCTAssertTrue([t.result isKindOfClass:[OOSGetBucketLoggingOutput class]]);
		
		OOSGetBucketLoggingOutput *output = t.result;
		OOSLoggingEnabled *loggingEnabled = output.loggingEnabled;
		XCTAssertTrue([loggingEnabled.targetBucket containsString:@"norris"]);
		XCTAssertTrue([loggingEnabled.targetPrefix containsString:@"nnn"]);
		XCTAssertTrue([loggingEnabled.triggerTargetBucket containsString:@"norris"]);
		XCTAssertTrue([loggingEnabled.triggerTargetPrefix containsString:@"mmm"]);
		XCTAssertTrue([loggingEnabled.triggerSourceBucket containsString:@"norris"]);
		XCTAssertTrue([loggingEnabled.triggerSourcePrefix containsString:@"bbb"]);
		
		return t;
	}] waitUntilFinished];
}

- (void)testBucketPolicy {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	NSDictionary *dic = @{
						  @"Statement": @[@{
											  @"Effect": @"Allow",
											  @"Principal": @{
													  @"AWS": @[@"*"]
													  },
											  @"Action": @"s3:*",
											  @"Resource": @"arn:aws:s3:::*/*"
											  }]
						  };
	
	OOSPutBucketPolicyRequest *putReq = [OOSPutBucketPolicyRequest new];
	putReq.bucket = TestAttrBucketName;
	putReq.policy = dic;
	
	[[[[[client putBucketPolicy:putReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		
		OOSGetBucketPolicyRequest *getReq = [OOSGetBucketPolicyRequest new];
		getReq.bucket = TestAttrBucketName;
		
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:20]];
		return [client getBucketPolicy:getReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		XCTAssertTrue([t.result isKindOfClass:[OOSGetBucketPolicyOutput class]]);
		
		OOSGetBucketPolicyOutput *output = t.result;
		NSLog(@"%@", output);
		
		OOSDeleteBucketPolicyRequest *deleteReq = [OOSDeleteBucketPolicyRequest new];
		deleteReq.bucket = putReq.bucket;
		
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return [client deleteBucketPolicy:deleteReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		
		return t;
	}] waitUntilFinished];
}

- (void)testBucketTrigger {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSRemoteSite *remoteSite = [OOSRemoteSite new];
	remoteSite.remoteEndPoint = @"https://oos-hz.ctyunapi.cn";
	remoteSite.replicaMode = @"EC_3_0";
	remoteSite.remoteBucketName = @"norris";
	remoteSite.remoteAK = @"2510935f75d781385b35";
	remoteSite.remoteSK = @"567b89b544a89bb3d3ccef4be8507e6d30d75579";
	
	OOSTrigger *trigger = [OOSTrigger new];
	trigger.triggerName = @"aaa";
	trigger.isDefault = @"true";
	trigger.remoteSite = remoteSite;
	
	OOSTriggerConfiguration *config = [OOSTriggerConfiguration new];
	config.triggers = @[trigger];
	
	OOSPutBucketTriggerRequest *putReq = [OOSPutBucketTriggerRequest new];
	putReq.bucket = TestAttrBucketName;
	putReq.triggerConfiguration = config;
	
	[[[[[client putBucketTrigger:putReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);

		OOSGetBucketTriggerRequest *getReq = [OOSGetBucketTriggerRequest new];
		getReq.bucket = TestAttrBucketName;

		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return [client getBucketTrigger:getReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);
		XCTAssertTrue([t.result isKindOfClass:[OOSGetBucketTriggerOutput class]]);

		OOSGetBucketTriggerOutput *output = t.result;
		OOSTrigger *trigger = [output.triggerConfiguration.triggers firstObject];
		XCTAssertTrue([trigger.triggerName isEqualToString:@"aaa"]);
		XCTAssertTrue([trigger.isDefault isEqualToString:@"true"]);

		OOSDeleteBucketTriggerRequest *deleteReq = [OOSDeleteBucketTriggerRequest new];
		deleteReq.bucket = putReq.bucket;
		deleteReq.triggerName = @"aaa";

		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
		return [client deleteBucketTrigger:deleteReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull t) {
		XCTAssertNil(t.error);

		return t;
	}] waitUntilFinished];
}

- (void)testPutGetAndDeleteObject {
    NSString *testObjectStr = @"a test object string.";
    NSString *keyName = @"ios-test-put-get-and-delete-obj";
    NSData *testObjectData = [testObjectStr dataUsingEncoding:NSUTF8StringEncoding];
    
    OOS *client = [OOS defaultOOS];
    XCTAssertNotNil(client);
    
    OOSPutObjectRequest *putObjectRequest = [OOSPutObjectRequest new];
    putObjectRequest.bucket = TestBucketName;
    putObjectRequest.key = keyName;
    putObjectRequest.body = testObjectData;
    putObjectRequest.expires = [NSDate dateWithTimeIntervalSinceNow:100];
    putObjectRequest.contentLength = [NSNumber numberWithUnsignedInteger:[testObjectData length]];
    putObjectRequest.contentType = @"text/html";
    
    //Add User Metadata
    NSDictionary *userMetaData = @{@"user-data-1": @"user-metadata-value1",
                                   @"user-data-2": @"user-metadata-value2",
                                   @"x-amz-storage-class": @"EC_2_1_2",
                                   };
    putObjectRequest.metadata = userMetaData;
    
    [[[[[client putObject:putObjectRequest] continueWithBlock:^id(OOSTask *task) {
        XCTAssertNil(task.error);
        XCTAssertTrue([task.result isKindOfClass:[OOSPutObjectOutput class]], @"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSPutObjectOutput class]), [task.result description]);
        OOSPutObjectOutput *putObjectOutput = task.result;
        XCTAssertNotNil(putObjectOutput.ETag);
        XCTAssertEqual(putObjectOutput.serverSideEncryption, OOSServerSideEncryptionUnknown);
        
        OOSGetObjectRequest *getObjectRequest = [OOSGetObjectRequest new];
        getObjectRequest.bucket = TestBucketName;
        getObjectRequest.key = keyName;
        
        return [client getObject:getObjectRequest];
    }] continueWithBlock:^id(OOSTask *task) {
        XCTAssertNil(task.error);
        XCTAssertTrue([task.result isKindOfClass:[OOSGetObjectOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSGetObjectOutput class]),[task.result description]);
        OOSGetObjectOutput *getObjectOutput = task.result;
        NSData *receivedBody = getObjectOutput.body;
        XCTAssertEqualObjects(testObjectData,receivedBody, @"received object is different from sent object, expect:%@ but got:%@",[[NSString alloc] initWithData:testObjectData encoding:NSUTF8StringEncoding],[[NSString alloc] initWithData:receivedBody encoding:NSUTF8StringEncoding]);
        
        XCTAssertEqualObjects(userMetaData, getObjectOutput.metadata, @"getObjectOutput doesn't contains the metadata we expected");
        
        OOSDeleteObjectRequest *deleteObjectRequest = [OOSDeleteObjectRequest new];
        deleteObjectRequest.bucket = TestBucketName;
        deleteObjectRequest.key = keyName;
        
        return [client deleteObject:deleteObjectRequest];
    }] continueWithBlock:^id(OOSTask *task) {
        XCTAssertNil(task.error);
        XCTAssertTrue([task.result isKindOfClass:[OOSDeleteObjectOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSDeleteObjectOutput class]),[task.result description]);
        return nil;
    }] waitUntilFinished];
}

- (void)testMultipartUploadWithComplete {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	NSTimeInterval timeIntervalSinceReferenceDate = [NSDate timeIntervalSinceReferenceDate];
	NSString *keyName = [NSString stringWithFormat:@"%@%lld", NSStringFromSelector(_cmd), (int64_t)timeIntervalSinceReferenceDate];

	NSMutableString *testString = [NSMutableString string];
	for (int32_t i = 0; i < 1000000; i++) {
		[testString appendFormat:@"%d", i];
	}
	
	NSData *testData = [testString dataUsingEncoding:NSUTF8StringEncoding];
	__block NSString *uploadId = @"";
	__block NSString *resultETag = @"";
	
	NSUInteger partCount = ceil((double)[testData length] / OOSTestsTransferManagerMinimumPartSize);
	NSMutableArray *completedParts = [NSMutableArray arrayWithCapacity:partCount];
	for (int32_t i = 0; i < partCount; i++) {
		[completedParts addObject:[NSNull null]];
	}
	
	OOSCreateMultipartUploadRequest *createReq = [OOSCreateMultipartUploadRequest new];
	createReq.bucket = TestBucketName;
	createReq.key = keyName;
	createReq.expires = [NSDate dateWithTimeIntervalSinceNow:100];
	
	[[[[[client createMultipartUpload:createReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		OOSCreateMultipartUploadOutput *output = task.result;
		XCTAssertTrue([task.result isKindOfClass:[OOSCreateMultipartUploadOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSCreateMultipartUploadOutput class]),[task.result description]);
		uploadId = output.uploadId;
		
		NSMutableArray *partUploadTasks = [NSMutableArray arrayWithCapacity:partCount];
		
		for (int32_t i = 1; i < partCount + 1; i++) {
			NSUInteger dataLength = i == partCount ? [testData length] - ((i - 1) * OOSTestsTransferManagerMinimumPartSize) : OOSTestsTransferManagerMinimumPartSize;
			NSData *partData = [testData subdataWithRange:NSMakeRange((i - 1) * OOSTestsTransferManagerMinimumPartSize, dataLength)];
			
			OOSUploadPartRequest *uploadPartRequest = [OOSUploadPartRequest new];
			uploadPartRequest.bucket = TestBucketName;
			uploadPartRequest.key = keyName;
			uploadPartRequest.partNumber = @(i);
			uploadPartRequest.body = partData;
			uploadPartRequest.contentLength = @(dataLength);
			uploadPartRequest.uploadId = uploadId;
			
			[partUploadTasks addObject:[[client uploadPart:uploadPartRequest] continueWithSuccessBlock:^id(OOSTask *task) {
				XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
				XCTAssertTrue([task.result isKindOfClass:[OOSUploadPartOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSUploadPartOutput class]),[task.result description]);
				OOSUploadPartOutput *partOuput = task.result;
				XCTAssertNotNil(partOuput.ETag);
				
				OOSCompletedPart *completedPart = [OOSCompletedPart new];
				completedPart.partNumber = @(i);
				completedPart.ETag = partOuput.ETag;
				
				[completedParts replaceObjectAtIndex:i - 1 withObject:completedPart];
				
				return nil;
			}]];
		}
		
		return [OOSTask taskForCompletionOfAllTasks:partUploadTasks];
	}] continueWithSuccessBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		
		for (id cp in completedParts) {
			XCTAssertTrue([cp isKindOfClass:[OOSCompletedPart class]]);
		}
		
		OOSCompleteMultipartUploadRequest *compReq = [OOSCompleteMultipartUploadRequest new];
		compReq.bucket = TestBucketName;
		compReq.key = keyName;
		compReq.uploadId = uploadId;
		
		return [client completeMultipartUpload:compReq];
	}] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSCompleteMultipartUploadOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSCompleteMultipartUploadOutput class]),[task.result description]);
		OOSCompleteMultipartUploadOutput *compOutput = task.result;
		resultETag = compOutput.ETag;
		
		XCTAssertNotNil(compOutput.location);
		XCTAssertEqualObjects(compOutput.bucket, TestBucketName);
		XCTAssertEqualObjects(compOutput.key, keyName);
		
		return nil;
	}] waitUntilFinished];
	
	[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:20]];
	
	OOSListObjectsRequest *listObjectReq = [OOSListObjectsRequest new];
	listObjectReq.bucket = TestBucketName;
	listObjectReq.prefix = NSStringFromSelector(_cmd);
	
	[[[client listObjects:listObjectReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListObjectsOutput class]],@"The response object is not a class of [%@]", NSStringFromClass([OOSListObjectsOutput class]));
		OOSListObjectsOutput *listObjectsOutput = task.result;
		XCTAssertEqualObjects(listObjectsOutput.name, TestBucketName);
		
		BOOL match = NO;
		for (OOSObject *Object in listObjectsOutput.contents) {
			if ([Object.key isEqualToString:keyName]) {
				match = YES;
			}
		}
		
		XCTAssertTrue(match, @"Didn't find the uploaded object in the bucket!");
		
		return nil;
	}] waitUntilFinished];
	
	OOSDeleteObjectRequest *deleteObjectRequest = [OOSDeleteObjectRequest new];
	deleteObjectRequest.bucket = TestBucketName;
	deleteObjectRequest.key = keyName;
	
	[[[client deleteObject:deleteObjectRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSDeleteObjectOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSDeleteObjectOutput class]),[task.result description]);
		return nil;
	}] waitUntilFinished];
}

- (void)testMultipartAbortCopyAndList {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	NSTimeInterval timeIntervalSinceReferenceDate = [NSDate timeIntervalSinceReferenceDate];
	NSString *keyName = [NSString stringWithFormat:@"%@%lld", NSStringFromSelector(_cmd), (int64_t)timeIntervalSinceReferenceDate];
	
	__block NSString *uploadId = @"";
	
	OOSCreateMultipartUploadRequest *createReq = [OOSCreateMultipartUploadRequest new];
	createReq.bucket = TestBucketName;
	createReq.key = keyName;
	
	[[[[[[client createMultipartUpload:createReq] continueWithBlock:^id _Nullable(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		OOSCreateMultipartUploadOutput *output = task.result;
		uploadId = output.uploadId;
		
		OOSUploadPartCopyRequest *copyReq = [OOSUploadPartCopyRequest new];
		copyReq.bucket = TestBucketName;
		copyReq.key = keyName;
		copyReq.partNumber = [NSNumber numberWithInteger:1];
		copyReq.uploadId = uploadId;
		copyReq.replicateSource = [NSString stringWithFormat:@"%@/%@", TestBucketName, @"temp.txt"];
		
		return [client uploadPartCopy:copyReq];
	}] continueWithSuccessBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSUploadPartCopyOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSUploadPartCopyOutput class]),[task.result description]);
		
		OOSUploadPartCopyOutput *copyOutput = task.result;
		XCTAssertNotNil(copyOutput.replicatePartResult);
		OOSReplicatePartResult *result = copyOutput.replicatePartResult;
		XCTAssertNotNil(result.ETag);
		XCTAssertNotNil(result.lastModified);
		
		OOSListMultipartUploadsRequest *listReq = [OOSListMultipartUploadsRequest new];
		listReq.bucket = TestBucketName;
		listReq.keyMarker = @"1";
		listReq.maxUploads = [NSNumber numberWithInteger:3];
		listReq.uploadIdMarker = uploadId;
		
		return [client listMultipartUploads:listReq];
	}] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListMultipartUploadsOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSListMultipartUploadsOutput class]),[task.result description]);
		OOSListMultipartUploadsOutput *listOutput = task.result;
		
		XCTAssertEqualObjects(listOutput.bucket, TestBucketName);
		XCTAssertEqualObjects(listOutput.uploadIdMarker, uploadId);
		XCTAssertTrue(listOutput.maxUploads.integerValue == 3);
		XCTAssertTrue([listOutput.uploads count] == 1);
		
		OOSAbortMultipartUploadRequest *abortReq = [OOSAbortMultipartUploadRequest new];
		abortReq.bucket = TestBucketName;
		abortReq.key = keyName;
		abortReq.uploadId = uploadId;
		
		return [client abortMultipartUpload:abortReq];
	}] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSAbortMultipartUploadOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSAbortMultipartUploadOutput class]),[task.result description]);
		
		return nil;
	}] waitUntilFinished];
	
	OOSDeleteObjectRequest *deleteObjectRequest = [OOSDeleteObjectRequest new];
	deleteObjectRequest.bucket = TestBucketName;
	deleteObjectRequest.key = keyName;
	
	[[[client deleteObject:deleteObjectRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		return nil;
	}] waitUntilFinished];
}

- (void)testDeleteMultipleObjects {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	unsigned char *largeData = malloc(OOSTest256KB);
	memset(largeData, 5, OOSTest256KB);
	NSData *testObjectData = [[NSData alloc] initWithBytesNoCopy:largeData length:OOSTest256KB];
	
	
	for (int i=0;i<5;i++) {
		NSString *keyName = [NSString stringWithFormat:@"key%d",i];
		OOSPutObjectRequest *putObjectRequest = [OOSPutObjectRequest new];
		putObjectRequest.bucket = TestBucketName;
		putObjectRequest.key = keyName;
		putObjectRequest.body = testObjectData;
		putObjectRequest.contentLength = [NSNumber numberWithUnsignedInteger:[testObjectData length]];

		[[[client putObject:putObjectRequest] continueWithBlock:^id(OOSTask *task) {
			XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
			return nil;
		}] waitUntilFinished];
	}

	[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:10]];
	
	NSMutableArray *objectsArray = [NSMutableArray new];
	for (int i=0;i<5;i++) {
		NSString *keyName = [NSString stringWithFormat:@"key%d",i];
		OOSObjectIdentifier *obj = [OOSObjectIdentifier new];
		obj.key = keyName;
		[objectsArray addObject:obj];
	}
	
	OOSRemove *Remove = [OOSRemove new];
	Remove.quiet = [NSNumber numberWithBool:false];
	Remove.objects = objectsArray;
	
	OOSDeleteObjectsRequest *multipleObjectsDeleteReq = [OOSDeleteObjectsRequest new];
	multipleObjectsDeleteReq.bucket = TestBucketName;
	multipleObjectsDeleteReq.remove = Remove;
	
	[[[client deleteObjects:multipleObjectsDeleteReq ] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		OOSDeleteObjectsOutput *output = task.result;
		XCTAssertEqual([OOSDeleteObjectsOutput class], [output class]);
		
		NSMutableSet *deletedSet = [NSMutableSet new];

		NSMutableSet *expectedSet = [NSMutableSet new];
		for (int i=0;i<5;i++) {
			[expectedSet addObject:[NSString stringWithFormat:@"key%d",i]];
			OOSDeletedObject *deletedObj = output.deleted[i];
			if (deletedObj.key != nil) {
				[deletedSet addObject:deletedObj.key];
			}
		}
		XCTAssertEqualObjects(expectedSet, deletedSet);
		
		return nil;
	}] waitUntilFinished];
	
}

- (void)testAccessKey {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	__block NSString *accessKeyId = nil;

	OOSCreateAccessKeyRequest *postReq = [OOSCreateAccessKeyRequest new];
	[[[[[[[client createAccessKey:postReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);

		OOSCreateAccessKeyOutput *output = task.result;
		XCTAssertEqual([OOSCreateAccessKeyOutput class], [output class]);

		accessKeyId = output.createAccessKeyResponse.createAccessKeyResult.accessKey.accessKeyId;

		OOSUpdateAccessKeyRequest *updateReq = [OOSUpdateAccessKeyRequest new];
		updateReq.accessKeyId = accessKeyId;
		updateReq.status = @"Inactive";
		updateReq.isPrimary = @"true";

		return [client updateAccessKey:updateReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull task) {

		XCTAssertNil(task.error);

		OOSUpdateAccessKeyOutput *output = task.result;
		XCTAssertEqual([OOSUpdateAccessKeyOutput class], [output class]);

		OOSListAccessKeyRequest *listReq = [OOSListAccessKeyRequest new];
		listReq.maxItems = [NSNumber numberWithInteger:10];

		return [client listAccessKey:listReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull task) {

		XCTAssertNil(task.error);

		OOSListAccessKeyOutput *output = task.result;
		XCTAssertEqual([OOSListAccessKeyOutput class], [output class]);

		BOOL found = false;
		for (OOSAccessKeyMember *member in output.listAccessKeysResponse.listAccessKeysResult.accessKeyMetadata.members) {
			if ( [member.accessKeyId isEqualToString:accessKeyId] ) {
				found = true;
				break;
			}
		}
		XCTAssertTrue(found, @"should find an member which accessKeyId is %@", accessKeyId);
		
		OOSDeleteAccessKeyRequest *deleteReq = [OOSDeleteAccessKeyRequest new];
		deleteReq.accessKeyId = accessKeyId;
		
		return [client deleteAccessKey:deleteReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull task) {
		XCTAssertNil(task.error);
		
		OOSDeleteAccessKeyOutput *output = task.result;
		XCTAssertEqual([OOSDeleteAccessKeyOutput class], [output class]);
		
		OOSListAccessKeyRequest *listReq = [OOSListAccessKeyRequest new];
		listReq.maxItems = [NSNumber numberWithInteger:10];
		
		return [client listAccessKey:listReq];
	}] continueWithBlock:^id _Nullable(OOSTask * _Nonnull task) {
		
		XCTAssertNil(task.error);
		
		OOSListAccessKeyOutput *output = task.result;
		XCTAssertEqual([OOSListAccessKeyOutput class], [output class]);
		
		BOOL found = false;
		for (OOSAccessKeyMember *member in output.listAccessKeysResponse.listAccessKeysResult.accessKeyMetadata.members) {
			if ( [member.accessKeyId isEqualToString:accessKeyId] ) {
				found = true;
				break;
			}
		}
		XCTAssertFalse(found, @"should not find an member which accessKeyId is %@", accessKeyId);
		
		return nil;
	}] waitUntilFinished];
}

- (void)testCopyObject {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSCopyObjectRequest *copyReq = [OOSCopyObjectRequest new];
	copyReq.bucket = TestBucketName;
	copyReq.key = @"copyObjectTest";
	copyReq.replicateSource = @"/norris/onepiece.jpg";
	
	[[[client copyObject:copyReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		
		OOSCopyObjectOutput *output = task.result;
		XCTAssertEqual([OOSCopyObjectOutput class], [output class]);
		
		return nil;
	}] waitUntilFinished];
}

@end
