//
//  OOSTransferManagerTests.m
//  OOSTests
//
//  Created by Ye Tong on 2018/11/11.
//  Copyright © 2018年 CTYun. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "OOSService.h"
#import "OOSTransferManager.h"

@interface OOSTransferManagerTests : XCTestCase

@end


@implementation OOSTransferManagerTests

static NSString *testBucketNameGeneral = @"norris";

static NSURL *tempLargeURL = nil;
static NSURL *tempSmallURL = nil;

+ (void)setUp {
	OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:@"2510935f75d781385b35" secretKey:@"567b89b544a89bb3d3ccef4be8507e6d30d75579"];
	OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionHangZhou
																		 credentialsProvider:provider];
	[OOSServiceManager defaultServiceManager].defaultServiceConfiguration = configuration;
	
	
	//Create a large temporary file for uploading & downloading test
	tempLargeURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@-tmTestTempLarge",testBucketNameGeneral]]];
	NSError *error = nil;
	if (![[NSFileManager defaultManager] createFileAtPath:tempLargeURL.path contents:nil attributes:nil]) {
		OOSDDLogError(@"Error: Can not create file with file path:%@",tempLargeURL.path);
	}
	error = nil;
	NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingToURL:tempLargeURL error:&error];
	if (error) {
		OOSDDLogError(@"Error: [%@]", error);
	}
	
	@autoreleasepool {
		NSMutableString *tempBaseString = [NSMutableString string];
		for (int32_t i = 0; i < 800000; i++) { //800000 = 4.68MB
			[tempBaseString appendFormat:@"%d", i];
		}
		
		int multiplier = 5;

		for (int32_t j = 0; j < multiplier; j++) {
			@autoreleasepool {
				[fileHandle writeData:[tempBaseString dataUsingEncoding:NSUTF8StringEncoding]];
			}
		}
		[fileHandle closeFile];
		
		if (true) {
			//Create a smal temporary file for uploading & downloading test
			tempSmallURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@-tmTestTempSmall",testBucketNameGeneral]]];
			NSError *error = nil;
			if (![[NSFileManager defaultManager] createFileAtPath:tempSmallURL.path contents:nil attributes:nil]) {
				OOSDDLogError(@"Error: Can not create file with file path:%@",tempSmallURL.path);
			}
			error = nil;
			NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingToURL:tempSmallURL error:&error];
			if (error) {
				OOSDDLogError(@"Error: [%@]", error);
			}
			
			[fileHandle writeData:[tempBaseString dataUsingEncoding:NSUTF8StringEncoding]]; //baseString 800000 = 4.68MB
			
			[fileHandle closeFile];
		}
		
	}

}

+ (void)tearDown {
	// Put teardown code here. This method is called after the invocation of each test method in the class.
	[super tearDown];
	
	//Delete Temp files
	if (tempLargeURL) {
		[[NSFileManager defaultManager] removeItemAtURL:tempLargeURL error:nil];
	}
	if (tempSmallURL) {
		[[NSFileManager defaultManager] removeItemAtURL:tempSmallURL error:nil];
	}
}

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testDefault {
	OOSTransferManager *TransferManager = [OOSTransferManager defaultTransferManager];
	XCTAssertNotNil(TransferManager);
}

- (void)testRegister {
	OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:@"2510935f75d781385b35" secretKey:@"567b89b544a89bb3d3ccef4be8507e6d30d75579"];

	OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionBeiJing2
																		 credentialsProvider:provider];

	[OOSTransferManager registerTransferManagerWithConfiguration:configuration forKey:@"bj2"];

	OOSTransferManager *TransferManager = [OOSTransferManager defaultTransferManager];
	XCTAssertNotNil(TransferManager);

	[OOSTransferManager removeTransferManagerForKey:@"bj2"];
}

- (void)testTMDownloadSmallSizeWithProgressFeedback {
	
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	
	OOSTransferManager *transferManager = [OOSTransferManager defaultTransferManager];
	XCTAssertNotNil(transferManager);
	
	
	//Upload a file to the bucket
	NSTimeInterval timeIntervalSinceReferenceDate = [NSDate timeIntervalSinceReferenceDate];
	NSString *keyName = [NSString stringWithFormat:@"%@%lld", NSStringFromSelector(_cmd), (int64_t)timeIntervalSinceReferenceDate];
	
	NSError *error = nil;
	XCTAssertTrue([[NSFileManager defaultManager] fileExistsAtPath:tempSmallURL.path]);
	NSString *fileName = [NSString stringWithFormat:@"%@-%lld",NSStringFromSelector(_cmd), (int64_t)timeIntervalSinceReferenceDate];
	NSURL *testDataURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:fileName]];
	[[NSFileManager defaultManager] createSymbolicLinkAtURL:testDataURL withDestinationURL:tempSmallURL error:&error];
	XCTAssertNil(error, @"The request failed. error: [%@]", error);
	
	NSDictionary *attributes = [[NSFileManager defaultManager] attributesOfItemAtPath:tempSmallURL.path																			error:&error];
	XCTAssertNil(error, @"The request failed. error: [%@]", error);
	unsigned long long fileSize = [attributes fileSize];
	
	
	OOSTransferManagerUploadRequest *uploadRequest = [OOSTransferManagerUploadRequest new];
	uploadRequest.bucket = testBucketNameGeneral;
	uploadRequest.key = keyName;
	uploadRequest.body = testDataURL;
	
	__block int64_t accumulatedUploadBytes = 0;
	__block int64_t totalUploadedBytes = 0;
	__block int64_t totalExpectedUploadBytes = 0;
	uploadRequest.uploadProgress = ^(int64_t bytesSent, int64_t totalBytesSent, int64_t totalBytesExpectedToSend) {
		
		OOSDDLogDebug(@"keyName:%@ bytesSent: %lld, totalBytesSent: %lld, totalBytesExpectedToSend: %lld",keyName,bytesSent,totalBytesSent,totalBytesExpectedToSend);
		accumulatedUploadBytes += bytesSent;
		totalUploadedBytes = totalBytesSent;
		totalExpectedUploadBytes = totalBytesExpectedToSend;
	};
	
	[[[transferManager upload:uploadRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSTransferManagerUploadOutput class]], @"The response object is not a class of [%@], got: %@", NSStringFromClass([NSURL class]),NSStringFromClass([task.result class]));
		return nil;
	}] waitUntilFinished];
	
	XCTAssertEqual(totalUploadedBytes, accumulatedUploadBytes, @"total of accumulatedUploadBytes is not equal to totalUploadedBytes");
	XCTAssertEqual(fileSize, totalUploadedBytes, @"totalUploaded Bytes is not equal to fileSize");
	XCTAssertEqual(fileSize, totalExpectedUploadBytes);
	
	[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:5]];
	
	//Download the same file from the bucket
	OOSTransferManagerDownloadRequest *downloadRequest = [OOSTransferManagerDownloadRequest new];
	downloadRequest.bucket = testBucketNameGeneral;
	downloadRequest.key = keyName;
	
	NSString *downloadFileName = [NSString stringWithFormat:@"%@-downloaded-%@",NSStringFromSelector(_cmd),testBucketNameGeneral];
	downloadRequest.downloadingFileURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:downloadFileName]];
	
	//Create a situation that there is a file has already existed on that downloadingFileURL Path
	NSString *getObjectFilePath = [[NSBundle bundleForClass:[self class]] pathForResource:@"credentials" ofType:@"json"];
	[[NSFileManager defaultManager] copyItemAtPath:getObjectFilePath toPath:downloadRequest.downloadingFileURL.path error:nil];
	
	__block int64_t accumulatedDownloadBytes = 0;
	__block int64_t totalDownloadedBytes = 0;
	__block int64_t totalExpectedDownloadBytes = 0;
	downloadRequest.downloadProgress = ^(int64_t bytesWritten, int64_t totalBytesWritten, int64_t totalBytesExpectedToWrite) {
		accumulatedDownloadBytes += bytesWritten;
		totalDownloadedBytes = totalBytesWritten;
		totalExpectedDownloadBytes = totalBytesExpectedToWrite;
		OOSDDLogDebug(@"keyName:%@ bytesWritten: %lld, totalBytesWritten: %lld, totalBytesExpectedtoWrite: %lld",NSStringFromSelector(_cmd), bytesWritten,totalBytesWritten,totalBytesExpectedToWrite);
	};
	
	[[[transferManager download:downloadRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSTransferManagerDownloadOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSTransferManagerDownloadOutput class]),NSStringFromClass([task.result class]));
		OOSTransferManagerDownloadOutput *output = task.result;
		NSURL *receivedBodyURL = output.body;
		XCTAssertTrue([receivedBodyURL isKindOfClass:[NSURL class]], @"The response object is not a class of [%@], got: %@", NSStringFromClass([NSURL class]),NSStringFromClass([receivedBodyURL class]));
		
		//Compare file content
		XCTAssertTrue([[NSFileManager defaultManager] contentsEqualAtPath:receivedBodyURL.path andPath:[[NSFileManager defaultManager] destinationOfSymbolicLinkAtPath:testDataURL.path error:nil]], @"received and sent file are different1");
		XCTAssertEqual([[[NSFileManager defaultManager] attributesOfItemAtPath:receivedBodyURL.path error:nil] fileSize], [[[NSFileManager defaultManager] attributesOfItemAtPath:[[NSFileManager defaultManager] destinationOfSymbolicLinkAtPath:testDataURL.path error:nil] error:nil] fileSize]);
		
		return nil;
		
	}] waitUntilFinished];
	
	XCTAssertEqual(totalDownloadedBytes, accumulatedDownloadBytes, @"accumulatedDownloadBytes is not equal to totalDownloadedBytes");
	XCTAssertEqual(fileSize, totalDownloadedBytes,@"total downloaded fileSize is not equal to uploaded fileSize");
	XCTAssertEqual(fileSize, totalExpectedDownloadBytes);
	
	
	//Delete the object
	OOSDeleteObjectRequest *deleteObjectRequest = [OOSDeleteObjectRequest new];
	deleteObjectRequest.bucket = testBucketNameGeneral;
	deleteObjectRequest.key = keyName;
	
	[[[client deleteObject:deleteObjectRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSDeleteObjectOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSDeleteObjectOutput class]),NSStringFromClass([task.result class]));
		return nil;
	}] waitUntilFinished];
	
}


- (void)testTMPauseAllandResumeAllTasks {
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	OOSTransferManager *transferManager = [OOSTransferManager defaultTransferManager];
	XCTAssertNotNil(transferManager);
	
	NSTimeInterval timeIntervalSinceReferenceDate = [NSDate timeIntervalSinceReferenceDate];
	NSString *keyName = [NSString stringWithFormat:@"%@%lld", NSStringFromSelector(_cmd), (int64_t)timeIntervalSinceReferenceDate];
	
	//Create multiple upload tasks
	NSError *error = nil;
	XCTAssertTrue([[NSFileManager defaultManager] fileExistsAtPath:tempSmallURL.path]);
	NSString *fileName = [NSString stringWithFormat:@"%@-%@",NSStringFromSelector(_cmd), keyName];
	NSURL *testDataURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:fileName]];
	[[NSFileManager defaultManager] createSymbolicLinkAtURL:testDataURL
										 withDestinationURL:tempSmallURL
													  error:&error];
	XCTAssertNil(error, @"The request failed. error: [%@]", error);
	NSDictionary *attributes = [[NSFileManager defaultManager] attributesOfItemAtPath:tempSmallURL.path
																				error:&error];
	XCTAssertNil(error, @"The request failed. error: [%@]", error);
	unsigned long long fileSize = [attributes fileSize];
	
	
	NSString *keyName1 = [NSString stringWithFormat:@"%@-large-%d",keyName, 1];
	OOSTransferManagerUploadRequest *uploadRequest1 = [OOSTransferManagerUploadRequest new];
	uploadRequest1.bucket = testBucketNameGeneral;
	uploadRequest1.key = keyName1;
	uploadRequest1.body = testDataURL;
	
	
	OOSTask *uploadTask1 = [[transferManager upload:uploadRequest1] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNotNil(task.error,@"Expect got 'Cancelled' Error, but got nil");
		XCTAssertEqualObjects(OOSTransferManagerErrorDomain, task.error.domain);
		XCTAssertEqual(OOSTransferManagerErrorPaused, task.error.code);
		
		return nil;
	}];
	
	NSString *keyName2 = [NSString stringWithFormat:@"%@-large-%d",keyName,2];
	OOSTransferManagerUploadRequest *uploadRequest2 = [OOSTransferManagerUploadRequest new];
	uploadRequest2.bucket = testBucketNameGeneral;
	uploadRequest2.key = keyName2;
	uploadRequest2.body = testDataURL;
	
	
	OOSTask *uploadTask2 = [[transferManager upload:uploadRequest2] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNotNil(task.error,@"Expect got 'Cancelled' Error, but got nil");
		XCTAssertEqualObjects(OOSTransferManagerErrorDomain, task.error.domain);
		XCTAssertEqual(OOSTransferManagerErrorPaused, task.error.code);
		
		return nil;
	}];
	
	
	//Create Multiple downloadTask
	OOSTransferManagerDownloadRequest *downloadRequest1 = [OOSTransferManagerDownloadRequest new];
	downloadRequest1.bucket = testBucketNameGeneral;
	downloadRequest1.key = @"temp.txt";
	
	NSString *downloadFileName = [NSString stringWithFormat:@"%@-downloaded-%@-%d",keyName,testBucketNameGeneral,1];
	downloadRequest1.downloadingFileURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:downloadFileName]];
	OOSTask *downloadTask1 = [[transferManager download:downloadRequest1] continueWithBlock:^id(OOSTask *task) {
		//Should return Cancelled Task Error
		XCTAssertNotNil(task.error, @"Expect got 'Cancelled' Error, but got nil");
		XCTAssertEqualObjects(OOSTransferManagerErrorDomain, task.error.domain);
		XCTAssertEqual(OOSTransferManagerErrorPaused, task.error.code);
		return nil;
	}];
	
	OOSTransferManagerDownloadRequest *downloadRequest2 = [OOSTransferManagerDownloadRequest new];
	downloadRequest2.bucket = testBucketNameGeneral;
	downloadRequest2.key = @"temp2.txt";
	
	
	NSString *downloadFileName2 = [NSString stringWithFormat:@"%@-downloaded-%@-%d",keyName,testBucketNameGeneral,2];
	downloadRequest2.downloadingFileURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:downloadFileName2]];
	OOSTask *downloadTask2 = [[transferManager download:downloadRequest2] continueWithBlock:^id(OOSTask *task) {
		//Should return Cancelled Task Error
		XCTAssertNotNil(task.error,@"Expect got 'Cancelled' Error, but got nil");
		XCTAssertEqualObjects(OOSTransferManagerErrorDomain, task.error.domain);
		XCTAssertEqual(OOSTransferManagerErrorPaused, task.error.code);
		return nil;
	}];
	
	
	//Wait a few second and then pause all tasks
	[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:2]];
	[[transferManager pauseAll] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error); //should not return error if successfully paused.
		return nil;
	}];
	
	//make sure all uploading/downloading task callback has been called.
	[uploadTask1 waitUntilFinished];
	[uploadTask2 waitUntilFinished];
	
	[downloadTask1 waitUntilFinished];
	[downloadTask2 waitUntilFinished];

	[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:5]];

	//resume it
	__block int expectResumeRequestCount = 0;
	__block int totalResumedRequestCount = 0;
	[[[transferManager resumeAll:^(OOSRequest *request) {
		totalResumedRequestCount++;

		if ([request isKindOfClass:[OOSTransferManagerUploadRequest class]]) {
			OOSTransferManagerUploadRequest *pausedUploadRequest = (OOSTransferManagerUploadRequest *)request;
			if ([pausedUploadRequest.key isEqualToString:keyName1]) {
				//upload request 1
				expectResumeRequestCount++;

			} else if ([pausedUploadRequest.key isEqualToString:keyName2]) {
				//upload request 2
				expectResumeRequestCount++;
			} else {
				XCTFail(@"unexpected uploadRequest received:\n %@",pausedUploadRequest);
			}

		} else if  ([request isKindOfClass:[OOSTransferManagerDownloadRequest class]]) {
			OOSTransferManagerDownloadRequest *pausedDownloadRequest =(OOSTransferManagerDownloadRequest *)request;
			if ([pausedDownloadRequest.key isEqualToString:@"temp.txt"]) {
				//download request 1
				expectResumeRequestCount++;
			} else if ([pausedDownloadRequest.key isEqualToString:@"temp2.txt"]) {
				//download request 2
				expectResumeRequestCount++;
			} else {
				XCTFail(@"unexpected downloadRequest received:\n %@",pausedDownloadRequest);
			}
		} else {
			XCTFail(@"unexpected request received:\n %@",request);
		}


	}] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error);
		NSArray *results = task.result;
		XCTAssertEqual(4, [results count]);

		for (id responseObject in results) {
			if ([responseObject isKindOfClass:[OOSTransferManagerDownloadOutput class]]) {
				OOSTransferManagerDownloadOutput *output = (OOSTransferManagerDownloadOutput *)responseObject;
				NSURL *receivedBodyURL = output.body;
				XCTAssertTrue([receivedBodyURL isKindOfClass:[NSURL class]], @"The response object is not a class of [%@], got: %@", NSStringFromClass([NSURL class]),NSStringFromClass([receivedBodyURL class]));

				//Compare file size
				NSError *error = nil;
				XCTAssertNil(error, @"The request failed. error: [%@]", error);
			}
		}

		return nil;
	}] waitUntilFinished];

	XCTAssertEqual(4, expectResumeRequestCount, @"number of expected request is not matched");
	XCTAssertEqual(expectResumeRequestCount, totalResumedRequestCount, @"total number resumed request is not equal to expected number of request");

	//check if uploaded object exists
	OOSListObjectsRequest *listObjectReq = [OOSListObjectsRequest new];
	listObjectReq.bucket = testBucketNameGeneral;

	[[[client listObjects:listObjectReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListObjectsOutput class]],@"The response object is not a class of [%@]", NSStringFromClass([OOSListObjectsOutput class]));
		OOSListObjectsOutput *listObjectsOutput = task.result;
		XCTAssertEqualObjects(listObjectsOutput.name, testBucketNameGeneral);

		BOOL match = NO;
		for (OOSObject *Object in listObjectsOutput.contents) {
			if ([Object.key isEqualToString:keyName1] || [Object.key isEqualToString:keyName2]) {
				if ([Object.size unsignedIntegerValue] == fileSize) {
					match = YES;
				} else {
					XCTFail(@"file size is different on the server. expected:%lu, but got: %lu",(unsigned long)fileSize,(unsigned long)[Object.size unsignedIntegerValue]);
				}
			}
		}

		XCTAssertTrue(match, @"Didn't find the uploaded object in the bucket!");

		return nil;
	}] waitUntilFinished];

	//Delete uploaded Object
	OOSDeleteObjectRequest *deleteObjectRequest = [OOSDeleteObjectRequest new];
	deleteObjectRequest.bucket = testBucketNameGeneral;
	deleteObjectRequest.key = keyName1;

	[[[client deleteObject:deleteObjectRequest] continueWithBlock:^id(OOSTask *task) {
		return nil;
	}] waitUntilFinished];

	OOSDeleteObjectRequest *deleteObjectRequest2 = [OOSDeleteObjectRequest new];
	deleteObjectRequest2.bucket = testBucketNameGeneral;
	deleteObjectRequest2.key = keyName2;

	[[[client deleteObject:deleteObjectRequest2] continueWithBlock:^id(OOSTask *task) {
		return nil;
	}] waitUntilFinished];
	
}

- (void)testTMUploadPauseAndResumeLargeSizeWithProgressFeedback {
	
	OOS *client = [OOS defaultOOS];
	XCTAssertNotNil(client);
	OOSTransferManager *transferManager = [OOSTransferManager defaultTransferManager];
	XCTAssertNotNil(transferManager);
	
	
	//Upload a file to the bucket
	NSTimeInterval timeIntervalSinceReferenceDate = [NSDate timeIntervalSinceReferenceDate];
	NSString *keyName = [NSString stringWithFormat:@"%@%lld", NSStringFromSelector(_cmd), (int64_t)timeIntervalSinceReferenceDate];
	
	NSError *error = nil;
	XCTAssertTrue([[NSFileManager defaultManager] fileExistsAtPath:tempLargeURL.path]);
	NSString *fileName = [NSString stringWithFormat:@"%lld", (int64_t)timeIntervalSinceReferenceDate];
	
	NSURL *testDataURL = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:fileName]];
	[[NSFileManager defaultManager] createSymbolicLinkAtURL:testDataURL withDestinationURL:tempLargeURL error:&error];
	XCTAssertNil(error, @"The request failed. error: [%@]", error);
	
	NSDictionary *attributes = [[NSFileManager defaultManager] attributesOfItemAtPath:tempLargeURL.path error:&error];
	XCTAssertNil(error, @"The request failed. error: [%@]", error);
	unsigned long long fileSize = [attributes fileSize];
	
	OOSTransferManagerUploadRequest *uploadRequest = [OOSTransferManagerUploadRequest new];
	uploadRequest.bucket = testBucketNameGeneral;
	uploadRequest.key = keyName;
	uploadRequest.body = testDataURL;
	
	
	
	__block int64_t accumulatedUploadBytes = 0;
	__block int64_t totalUploadedBytes = 0;
	__block int64_t totalExpectedUploadBytes = 0;
	uploadRequest.uploadProgress = ^(int64_t bytesSent, int64_t totalBytesSent, int64_t totalBytesExpectedToSend) {
//		OOSDDLogDebug(@"keyName:%@ bytesSent: %lld, totalBytesSent: %lld, totalBytesExpectedToSend: %lld",keyName,bytesSent,totalBytesSent,totalBytesExpectedToSend);
		accumulatedUploadBytes += bytesSent;
		totalUploadedBytes = totalBytesSent;
		totalExpectedUploadBytes = totalBytesExpectedToSend;
	};
	
	XCTAssertEqual(uploadRequest.state, OOSTransferManagerRequestStateNotStarted);
	
	[[transferManager upload:uploadRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNotNil(task.error,@"Expect got 'Cancelled' Error, but got nil");
		XCTAssertEqualObjects(OOSTransferManagerErrorDomain, task.error.domain);
		XCTAssertEqual(OOSTransferManagerErrorPaused, task.error.code);
		if ([task.error.domain isEqualToString:OOSTransferManagerErrorDomain] == NO || task.error.code != OOSTransferManagerErrorPaused) {
			OOSDDLogDebug(@"unexpected error:%@",task.error);
		}
		return nil;
	}];
	
	XCTAssertEqual(uploadRequest.state, OOSTransferManagerRequestStateRunning);
	
	//random pause and resume task until it finished
	OOSTask *currentTask = nil;
	__block BOOL isFinished = NO;
	for (int32_t i=0; i<5; i++) {
		if (isFinished) break;
		int randNum = rand() % (8 - 3) + 3; //create the random number between 3 to 8.
		//wait a random moment and pause the task
		OOSDDLogDebug(@"-------- Pause the Task in %d seconds--------",randNum);
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:randNum]];
		OOSDDLogDebug(@"-------- Pausing the Task --------");
		if (isFinished) break;
		[[[uploadRequest pause] continueWithBlock:^id(OOSTask *task) {
			XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error); //should not return error if successfully paused.
			return nil;
		}] waitUntilFinished];
		
		XCTAssertEqual(uploadRequest.state, OOSTransferManagerRequestStatePaused);
		
		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:3]];
		OOSDDLogDebug(@"-------- Resume the Task --------");
		//resume the upload
		if (isFinished) break;
		currentTask = [[transferManager upload:uploadRequest] continueWithBlock:^id(OOSTask *task) {
			if (task.error) {
				//if contains error, should be PauseError
				XCTAssertEqualObjects(OOSTransferManagerErrorDomain, task.error.domain);
				XCTAssertEqual(OOSTransferManagerErrorPaused, task.error.code);
				if ([task.error.domain isEqualToString:OOSTransferManagerErrorDomain] == NO || task.error.code != OOSTransferManagerErrorPaused) {
					OOSDDLogDebug(@"unexpected error:%@",task.error);
					isFinished = YES;
				}
			} else {
				isFinished = YES;
				XCTAssertTrue([task.result isKindOfClass:[OOSTransferManagerUploadOutput class]], @"The response object is not a class of [%@], got: %@", NSStringFromClass([NSURL class]),NSStringFromClass([task.result class]));
			}
			
			return nil;
		}];
		
		XCTAssertEqual(uploadRequest.state, OOSTransferManagerRequestStateRunning);
	}
	
	[currentTask waitUntilFinished];
	XCTAssertTrue(isFinished);
	
	XCTAssertEqual(uploadRequest.state, OOSTransferManagerRequestStateCompleted);
	
	//XCTAssertEqual(fileSize, accumulatedUploadBytes, @"total of accumulatedUploadBytes is not equal to fileSize");
	XCTAssertEqual(fileSize, totalUploadedBytes, @"totalUploaded Bytes is not equal to fileSize");
	XCTAssertEqual(fileSize, totalExpectedUploadBytes);
	
	[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:5]];
	
	OOSListObjectsRequest *listObjectReq = [OOSListObjectsRequest new];
	listObjectReq.bucket = testBucketNameGeneral;
	
	[[[client listObjects:listObjectReq] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSListObjectsOutput class]],@"The response object is not a class of [%@]", NSStringFromClass([OOSListObjectsOutput class]));
		OOSListObjectsOutput *listObjectsOutput = task.result;
		XCTAssertEqualObjects(listObjectsOutput.name, testBucketNameGeneral);
		
		BOOL match = NO;
		for (OOSObject *Object in listObjectsOutput.contents) {
			if ([Object.key isEqualToString:keyName]) {
				if ([Object.size unsignedIntegerValue] == fileSize) {
					match = YES;
				} else {
					XCTFail(@"file size is different on the server. expected:%lu, but got: %lu",(unsigned long)fileSize,(unsigned long)[Object.size unsignedIntegerValue]);
				}
			}
		}
		
		XCTAssertTrue(match, @"Didn't find the uploaded object in the bucket!");
		
		return nil;
	}] waitUntilFinished];
	
	OOSDeleteObjectRequest *deleteObjectRequest = [OOSDeleteObjectRequest new];
	deleteObjectRequest.bucket = testBucketNameGeneral;
	deleteObjectRequest.key = keyName;
	
	[[[client deleteObject:deleteObjectRequest] continueWithBlock:^id(OOSTask *task) {
		XCTAssertNil(task.error, @"The request failed. error: [%@]", task.error);
		XCTAssertTrue([task.result isKindOfClass:[OOSDeleteObjectOutput class]],@"The response object is not a class of [%@], got: %@", NSStringFromClass([OOSDeleteObjectOutput class]),NSStringFromClass([task.result class]));
		return nil;
	}] waitUntilFinished];
	
}


@end
