#oos-sdk-ios

## 进度

- [x] 初始化工程
- [x] 基础网络库
- [x] 解决断网/重新联网问题
- [x] 解析XML
- [x] 用户签名验证
- [x] Bucket权限控制
- [x] Policy安全策略
- [x] 单元测试框架
- [x] 实现TransferManager
- [x] 支持多Region
- [x] AccessKey
- [x] 支持分片上传
- [x] 支持Framework
- [x] 整合CocoaPods
- [x] 支持Swift调用

## 导入Framework
```objective-c
// 1. 在工程中点击EmbededFramework，勾选copy if needed，选中OOS.framework点击确定；
// 2. 拷贝trim.sh到工程目录下；
// 3. 打开XCode，选择Target，在Build Phases最后添加编译脚本，输入${SRCROOT}/trim.sh
// 4. 导入头文件
#import "OOS/OOS.h"

```


## 导入开发工具包 OSS API
```objective-c
// 将整个CTYun文件夹拖入工程中
#import "OOS.h"
```


## 设置accessKey和secretKey
```objective-c

// 可以在网页中获取secretKey
OOSStaticCredentialsProvider *provider = [[OOSStaticCredentialsProvider alloc] initWithAccessKey:@"xxx" secretKey:@"yyy"];

// 选择所属Region
OOSServiceConfiguration *configuration = [[OOSServiceConfiguration alloc] initWithRegion:OOSRegionHangZhou
credentialsProvider:provider];

// 设为默认配置
[OOSServiceManager defaultServiceManager].defaultServiceConfiguration = configuration;

```



## 文件的上传与下载
从存储桶中下载文件和将文件上传到存储桶，使用的是同样的编码模式。重要区别在于，除非下载完成，否则 download: 不会成功，这会阻止依赖这种成功的任何流。上传会立即返回，因此可在主线程上安全调用。

为传输调用 TransferManager 的步骤如下所示。

### 创建 TransferManager 客户端
```objective-c
OOSTransferManager *transferManager = [OOSTransferManager defaultTransferManager];
```

### 下载Object到本地
```objective-c
NSString *downloadingFilePath = [NSTemporaryDirectory() stringByAppendingPathComponent:@"myImage.jpg"];
NSURL *downloadingFileURL = [NSURL fileURLWithPath:downloadingFilePath];

OOSTransferManagerDownloadRequest *downloadRequest = [OOSTransferManagerDownloadRequest new];

downloadRequest.bucket = @"myBucket";
downloadRequest.key = @"myImage.jpg";
downloadRequest.downloadingFileURL = downloadingFileURL;

[[transferManager download:downloadRequest] continueWithBlock:^id(OOSTask *task) {
    if (task.error){
        NSLog(@"Error: %@", task.error);
    }

    if (task.result) {
        OOSTransferManagerDownloadOutput *downloadOutput = task.result;
    }
    return nil;
}];

```

### 上传文件到OSS
```objective-c
NSURL *uploadingFileURL = [NSURL fileURLWithPath: @"your/file/path/myTestFile.txt"];

OOSTransferManagerUploadRequest *uploadRequest = [OOSTransferManagerUploadRequest new];

uploadRequest.bucket = @"myBucket";
uploadRequest.key = @"myTestFile.txt";
uploadRequest.body = uploadingFileURL;

[[transferManager upload:uploadRequest] continueWithBlock:^id(OOSTask *task) {
	if (task.error) {
		NSLog(@"Error: %@", task.error);
	}

	if (task.result) {
		OOSTransferManagerUploadOutput *uploadOutput = task.result;
		// The file uploaded successfully.
	}
	return nil;
}];
```

### 在 UIImageView 中显示下载的图像

```objective-c
dispatch_async(dispatch_get_main_queue(), ^{
	self.imageView.image = [UIImage imageWithContentsOfFile:downloadingFilePath];
});
```

## 暂停、恢复和取消对象传输

### 使用 continueWith 块来处理结果

```objective-c
[[self.uploadRequest pause] continueWithBlock:^id(OOSTask *task) {
    if (task.error) {
        NSLog(@"Error: %@",task.error);
    } else {

    }

    // Upload has been paused.
    return nil;
}];
```

### 暂停传输

```objective-c
[uploadRequest pause];
[downloadRequest pause];
```


### 恢复传输

```objective-c
[transferManager upload:uploadRequest];
[transferManager download:downloadRequest];
```

### 取消传输

```objective-c
[uploadRequest cancel];
[downloadRequest cancel];
```

### 暂停所有传输

```objective-c
[transferManager pauseAll];
```

### 恢复所有传输

```objective-c
[transferManager resumeAll:^(OOSRequest *request) {
    // All paused requests have resumed.
}];
```

### 恢复所有传输

```objective-c
[transferManager cancelAll];
```


## 跟踪进度

利用 uploadProgress 和 downloadProgress 块，您可以跟踪对象传输的进度。这些块可与 Grand Central Dispatch dispatch_async 函数结合使用，如以下示例所示。

### 上传进度

```objective-c
uploadRequest.uploadProgress =  ^(int64_t bytesSent, int64_t totalBytesSent, int64_t totalBytesExpectedToSend){
    dispatch_async(dispatch_get_main_queue(), ^{
		 //Update progress
	});
};
```

### 下载进度

```objective-c
downloadRequest.downloadProgress = ^(int64_t bytesWritten, int64_t totalBytesWritten, int64_t totalBytesExpectedToWrite){
	dispatch_async(dispatch_get_main_queue(), ^{
			//Update progress
	});
};
```

## 关于Bucket的操作

### 获取所有Bucket

```objective-c

OOS client* = [OOS defaultOOS];
OOSListBucketsRequest *listBucketsRequest = [OOSListBucketsRequest new];
[client listBuckets:listBucketsRequest completionHandler: ^(OOSListBucketsOutput * response, NSError * error) {
  NSArray *buckets = response.buckets;
}];

```
